# Derive metadata for all csv files in this folder.
# Directions: Run each time updates are made to the ITHIM TOOL data (i.e. to
# the CSV files in /tool_files.)
# NOTICE: Backup the data_dictionary.csv prior to running this code!


# Load `digest` (or install).
if(!("digest" %in% .packages()))  # is it already loaded in workspace?
  if(require(digest) ==  F) {  # load it, but if you can't...
    install.packages("digest")  # install it.
    library(digest)  # try loading it again.
  }

# get list of csv's.
Filename <- list.files("tool_files")[grepl("\\.csv", 
                                       list.files("tool_files"), 
                                       perl = T)]

# is the data_dictionary.csv present? (T/F)
dd <- T %in% grepl("data\\_dictionary\\.csv", Filename, perl=T)

# remove data dictionary
if (dd == T)  # note, `dd` is used later as well.
  Filename <- Filename[!grepl("data\\_dictionary\\.csv", Filename, perl=T)]

# read all csvs & rename list
csv_l <- lapply(file.path("tool_files", Filename), function(x){
  read.csv(x, stringsAsFactors = F)
})
# rename list elements
names(csv_l) <- Filename

# object sizes
Var_Size <- sapply(csv_l, function(x) {object.size(x)})

# create majority of metadata (a list of df with one row each.)
metadata <- lapply(csv_l, function(x){
  data.frame(
    Code_Levels = paste(names(x), collapse=", "),
    Format = paste(sapply(x, function(y){typeof(y)}), collapse=", "),
    Num_Rows = nrow(x),
    Num_Features = ncol(x),
    Sha1 = digest::sha1(x),
    Timestamp = Sys.time(),
    stringsAsFactors = F,
    row.names = NULL
  )
})

# take list of data.frames (w/ 1 row each) -> 1 df.
metadata <- do.call(rbind, c(metadata, make.row.names=F))

# attach other columns.
metadata <- cbind(data.frame("Filename"=Filename, 
                             stringsAsFactors = F,
                             row.names=NULL), 
                  metadata, 
                  data.frame("Var_Size"=Var_Size, 
                             stringsAsFactors = F, 
                             row.names=NULL))

# Saving the Data Dictionary
if(dd==F) {
  # No data_dictionary.csv, thus create it.
  write.csv(metadata, 
            file.path("tool_files", "data_dictionary.csv"), 
            row.names=F)
  writeLines(text=c("Log for data_dictionary.csv",
                    paste0("Created ", Sys.time()),
                    "~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n"),
             con=file.path("tool_files", "data_log.txt"))
  # garbage collection
  rm(dd,
     Filename,
     metadata,
     Var_Size,
     csv_l)
} else {
  # data_dictionary.csv present, thus overwrite relevant rows.
  dd <- read.csv(file.path("tool_files", "data_dictionary.csv"), 
                 stringsAsFactors = F)
  
  # Get column names of dd and metadata.
  dd_cols <- colnames(dd)
  md_cols <- colnames(metadata)
  missing_cols <- dd_cols[!(dd_cols %in% md_cols)]
  original_timestamp <- c()
  
  # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  # Log/Print notice of changes.
  # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  
  # looking at changes from previous data_dictionary.csv
  log <- sapply(seq_len(nrow(dd)), function(x){
    current <- dd[x,"Filename"]
    if(current %in% metadata$Filename){
      # compare Sha1 values.
      check <- identical(dd[x,"Sha1"], 
                         metadata[which(metadata$Filename==current), "Sha1"])
      if(check == F){
        paste0("MODIFIED: ", current)
      } else {
        assign("original_timestamp", 
               c(original_timestamp,
                 current), 
               envir=parent.frame(3))
        NULL
      }
    } else {
      # Since it appears this csv is no longer present
      paste0("DELETED: ", current)
    }
  })
  
  # excise NULL values and simplify.
  log <- unlist(log[sapply(log, function(x) {!is.null(x)})])
  
  # add notice of new csv files from metadata.
  log2 <- metadata$Filename[!(metadata$Filename %in% dd$Filename)]
  log2 <- sapply(log2, function(x){
    paste0("ADDED: ", x)
  })
  log <- c(log, log2)
  
  # retrieve current log
  log_txt <- readLines(con=file.path("tool_files", "data_log.txt"))
  
  # give warning and add to log txt
  for(i in seq_along(log)){
    warning(log[i])
    log_txt <- c(log_txt, paste0("[", Sys.time(), "] ", log[i]))
  }
  
  # save log
  writeLines(text = log_txt,
             con=file.path("tool_files", "data_log.txt"))
  
  # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  # subset from dd "key" column and columns not present in metadata.
  dd_subset <- dd[,c("Filename", missing_cols)]
  
  # Update `metadata` with columns from
  # `dd` that are not present in
  # `metadata`. Keep only rows in
  # `metadata`
  metadata <- merge(
    x = metadata,
    y = dd_subset,
    by = "Filename",
    all.x = T,  # keep all rows from metadata
    all.y = F  # drop extra rows from dd
  )
  
  # Replace timestamp w/ original if the SHA1 same. (see `log <- sapply(...`)
  for(i in original_timestamp){
    metadata$Timestamp[which(metadata$Filename == i)] <- 
      dd$Timestamp[which(dd$Filename == i)]
  }
  
  # By chance if it is blank, just set current datetime.
  if(T %in% is.na(metadata$Timestamp))
    for(i in seq_len(nrow(metadata)))
      if(is.na(metadata$Timestamp[i]))
        metadata$Timestamp[i] <- Sys.time()
  
  if(!("ITHIM_Critical" %in% colnames(metadata)))
    metadata$ITHIM_Critical <- T
  
  # Is there a Variable_Name Column?
  if("Variable_Name" %in% dd_cols) {
    # Yes.
    # give users a heads up they need to add a Variable_Name
    if(T %in% is.na(metadata$Variable_Name)) {
      warning(paste("New csv files added!", 
                    "If you want it loaded, make sure to add a Variable_Name!"))
      # overwrite NA as ""
      metadata[is.na(metadata$Variable_Name), "Variable_Name"] <- ""
    }
    
    # reorder metadata so Variable_Name is first column.
    md_cols <- colnames(metadata)[!colnames(metadata)=="Variable_Name"]
    metadata <- metadata[c("Variable_Name", md_cols)]
  } else {
    # No.
    # create Variable_Name column in metadata
    metadata <- cbind(data.frame("Variable_Name"= rep("", nrow(metadata)),
                                 stringsAsFactors = F,
                                 row.names=NULL),
                      metadata)
  }
  # save metadata as data_dictionary.csv
  write.csv(metadata, 
            file.path("tool_files", "data_dictionary.csv"), 
            row.names=F)
  
  # garbage collection
  rm(dd,
     dd_cols,
     dd_subset,
     Filename,
     md_cols,
     metadata,
     missing_cols,
     Var_Size,
     csv_l,
     log,
     log2,
     i,
     log_txt,
     original_timestamp)
}