# Pre-Processing Code for ITHIM Tool
# November 2018 - Tomari/Maizlish
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# ////////////////////////////////////////////////////////////////////////////

# Note, the variable pth is defined in global.R
# pth <- "tool_files"

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# ITHIM Section ----
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Load Data ----
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Load data dictionary.
tmp <- read.csv(file.path(pth, "data_dictionary.csv"),
                stringsAsFactors = F)

# Check data dictionary for key columns
if(!("Variable_Name" %in% colnames(tmp) & 
   "Filename" %in% colnames(tmp))) stop("data dictionary broken.")

# Weed out unused variables (ie. lines without 'Variable_Name')
tmp <- tmp[which(!(tmp$Variable_Name=="")),]
rownames(tmp) <- NULL  # remove row numbers

# Extract narrative distances from tmp.
# Baseline & Scenario distances (ie. narratives)
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Note, where the Long Version expects
# multiple distances files. In this
# version, all scenarios are placed in
# a single csv file.
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
tmp2 <- tmp[grep("default_narratives", tmp$Variable_Name),]
# Everything but narratives.
tmp3 <- tmp[-grep("default_narratives", tmp$Variable_Name),]

# Check to make sure only a single narrative csv.
if(nrow(tmp2) > 1) stop("There's more than one default narrative csv!")

# Load default narratives
tmp2 <- read.csv(file.path(pth, tmp2$Filename[1]), stringsAsFactors = F)

# Rename list elements by the correct variable name.
default_narratives <- fn_narrativeSplitter(tmp2)  # former arg: default

# Load all (non-narrative) data to default data.
d <- lapply(tmp3$Filename, function(x) read.csv(file.path(pth, x), 
                                              stringsAsFactors = F))
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Note (this is an aside): fn_report_calc is expecting columns with "T" and
# "F" values are converted to a boolean type column. If this changes, please
# attend to the issue of the if statement used to identify "County" text in
# row icon_Walk.png.
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

# Rename list elements by the correct variable name.
names(d) <- tmp3$Variable_Name

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Verify Data Integrity
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Here, we utilize the column "Sha1" to compare the hash code established in
# the data_dictionary.csv to the hash of the data we're loading right now.
# Then we use "ITHIM_Critical" to generate a warning message or fail the ITHIM
# TOOL from functioning, or green light everything.
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

# data_integrity <- lapply(seq_along(d), function(x){
#   var_nm <- names(d[x])
#   original_sha <- tmp[which(tmp$Variable_Name==var_nm), "Sha1"]
#   old_sha <- digest::sha1(d[[x]])
#   comp <- identical(original_sha, old_sha)
#   critical <- tmp[which(tmp$Variable_Name==var_nm), "ITHIM_Critical"]
#   if(comp == F & critical == T)
#     paste0("Fatal Error. ",
#          "Please contact the system administrator ",
#          "with the following message: ",
#          "Data for \"",
#          var_nm, 
#          "\" is corrupted."
#          )
# else if (comp == F & critical == F)
#   paste0("Warning. ",
#            "Please contact the system administrator ",
#            "with the following message: ",
#            "Data for \"",
#            var_nm,
#            "\" does not match the record.")
#   else
#     NULL
# })

# data_integrity <- data_integrity[sapply(data_integrity, 
#                                        function(x) !is.null(x))]

# d$data_integrity <- data_integrity

# Garbage Collection.
rm(
  tmp, 
  tmp2,
  tmp3
#  data_integrity
  )  

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Create Scenario Selection List (Main Choice)
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Here we use this complicated
# approach to re-ordering the "pick
# list" because I wanted to allow for
# flexibility in how future
# programmers might include their own
# default baselines and scenarios.
# Now it mimics About > Scenarios
# ordering of narratives.
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
d$main_choice <- names(default_narratives)
d$main_choice <- d$main_choice[!grepl("baseline", d$main_choice, ignore.case=T)]
d$main_choice <- c(d$main_choice, 
                   "Caltrans Strategic Management Plan (2020)")
# sort alphabetically.
d$main_choice <- d$main_choice[order(d$main_choice)]
d$main_choice <- c("CARB Scoping Plan (2030)", d$main_choice)
# add last options
d$main_choice <- c(d$main_choice,
                   "Baseline Multiples",
                   "Fixed Time",
                   "Low Carbon Driving")
# re-order so short trips second to last.
tmp <- "Short Car Trips"
if(T %in% grepl(tmp, d$main_choice)){
  short_trips <- d$main_choice[grepl(tmp, d$main_choice)]
  d$main_choice <- d$main_choice[!grepl(tmp, d$main_choice)]
  d$main_choice <- c(d$main_choice, short_trips)
  rm(short_trips, tmp)
}
d$main_choice <- c(d$main_choice, "User Upload")

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Disease/Causes Lists ----
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Physical Activity only
d$pacauselist <- d$causes[d$causes$Pathway=="PA","Cause"]
d$paflist <- d$causes[d$causes$Pathway=="PA","dfcause"]

# PM only
d$pmcauselist <- d$causes[d$causes$Pathway=="PM","Cause"]
d$pmflist <- d$causes[d$causes$Pathway=="PM","dfcause"]

# PA & PM 
d$papmcauselist <- d$causes[d$causes$Pathway=="PA+PM","Cause"]
d$papmflist <- gsub("\\s", 
                     "", 
                    d$causes[d$causes$Pathway=="PA+PM","dfcause"], 
                    perl=TRUE)
# TODO fix papmflist's All Causes (note space)

# Road Traffic Injuries
d$rcauselist <- d$causes[d$causes$Pathway=="RTI","Cause"]
d$rflist <- d$causes[d$causes$Pathway=="RTI","dfcause"]

#Pathways
d$plist <- c("PA", "PM", "RTI","PA+PM")

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Amount of car miles substitution for walk and bike
# d$p <- 1

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Non-ITHIM Section ----
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# load data.frame (modified version of webtext.csv)
# d$txt  <-
#   read.csv(file = file.path(pth, "ithim_tool_strings.csv"),
#            stringsAsFactors = FALSE)

# Create data.frame for scenarios info box/tool tips
d$info_box <-
  do.call(rbind,
          lapply(
            # obtain text from d$txt, split by three semi-colons
            strsplit(d$txt[d$txt$Element == "info_box_text", "Content"],
                     "\\;{3}",
                     perl = T)[[1]],
            function(x) {
              # then split by three colons
              tmp <- strsplit(x, "\\:{3}", perl = T)[[1]]
              # place the output of above into a data.frame. Later rbind this.
              data.frame(scenario = tmp[1],
                         info = tmp[2],
                         stringsAsFactors = F)
            }))

# Create links that are embedded into text (eg for infobox above)
d$links <- 
  do.call(rbind,
          lapply(
            # obtain text from d$txt, split by three semi-colons
            strsplit(d$txt[d$txt$Element == "links", "Content"],
                     "\\;{3}",
                     perl = T)[[1]],
            function(x) {
              # then split by three colons
              tmp <- strsplit(x, "\\:{3}", perl = T)[[1]]
              # place the output of above into a data.frame. Later rbind this.
              data.frame(regex = tmp[1],
                         url = tmp[2],
                         stringsAsFactors = F)
            }))

# Embed links into info_box
d$info_box <- 
  do.call(rbind,
          lapply(split(d$info_box, d$info_box$scenario),
                 function(ro) {
                   # append More Info text, and place in <p> tags
                   ro$info <- paste0(
                     "<p>",
                     ro$info,
                     ' For more information, please visit the About > Scenarios page.',
                     "</p>"
                   )
                   for (i in seq_len(nrow(d$links))) {
                     if (grepl(pattern = d$links[i, "regex"],
                               x = ro$info)) {
                       tmp <- strsplit(x = ro$info,
                                       split = d$links[i, "regex"])[[1]]
                       ro$info <- paste0(tmp[1],
                                         '<a target="_blank"',
                                         'rel="noopener noreferrer" href="',
                                         d$links[i, "url"],
                                         '">',
                                         d$links[i, "regex"],
                                         '</a>',
                                         tmp[2])
                     }
                   }
                   
                   # return
                   ro
                 }))


# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Add panels (PNG files) for infographic.
d$infographic_pngs <- c(
  "01 - Header.png",
  "02 - Transportation Impact.png",
  "03 - Direct Impacts.png",
  "04 - Scenario Summary.png",
  "05 - GHG Emissions.png",
  "06 - Years of Life.png",
  "07 - Health Cost Savings.png",
  "08 - Air Pollution.png",
  "09 - Outcomes.png",
  "10 - Footnotes.png"
)

d$infographic_pngs <- sapply(d$infographic_pngs, function(x){
  png::readPNG(file.path(pth, x), native=T)
})

# save width
d$ig_w <- dim(d$infographic_pngs[[1]])[2]

# save all heights as a vector
d$ig_h <- sapply(seq_along(d$infographic_pngs), function(i){ 
  dim(d$infographic_pngs[[i]])[1]
  })